function pvfDFnl(fun,gun,hun,p1,p2,v1,v2,n,m,tol,wun)
% Solucin numrica de P.V.F no lineal
%      y''=f(x,y,y')   a<= x <=b 
%      y(a)=v1         y(b)=v2
% por el mtodo de diferencia finita.
% pvfDFnl('fun','gun','hun',p1,p2,v1,v2,n,m,tol,'wun')
% fun= f(x,y,y').
% gun= derivada con respecto a y de f(x,y,y'). 
% hun= derivada con respecto a y' de f(x,y,y').
% wun= solucin exacta del P.V.F.
% p1=a    p2=b
% n= nmero de subintervalos 
% m= nmero mximo de iteraciones
% tol= tolerancia
% 
% Ejemplos
% pvfDFnl('df1','df11','df12',1,3,17,43/3,20,10,1.e-8,'ex1')
% pvfDFnl('df2','df21','df22',1,4,1/3,20/3,30,10,1.e-8,'ex2')
% pvfDFnl('df3','df31','df32',0,2,pi/2,pi/4,20,10,1.e-8,'ex3')
% pvfDFnl('df4','df41','df42',1,2,4,8,20,10,1.e-8,'ex4')
% pvfDFnl('df5','df51','df52',1,2,0,log(2),20,10,1.e-8,'ex5')
%fun;
%gun;
%hun;
%wun;
a=zeros(n-1,1);b=zeros(n-2,1);c=zeros(n-2,1);
d=zeros(n-1,1);
h=(p2-p1)/n;k=1;pe=(v2-v1)/(p2-p1);
x=p1:h:p2;w=feval(wun,x');
y=v1+pe*(x'-p1);
while k<=m
   for i=1:n-1
      t(i)=(y(i+2)-y(i))/(2*h);
      a(i)=2+(h^2)*feval(gun,x(i+1),y(i+1),t(i));
      d(i)=y(i)-2*y(i+1)+y(i+2)-(h^2)*feval(fun,x(i+1),y(i+1),t(i));
   end
   for i=1:n-2
      b(i)=-1+(h/2)*feval(hun,x(i+1),y(i+1),t(i));
      c(i)=-1-(h/2)*feval(hun,x(i+2),y(i+2),t(i+1));
   end
   J=diag(a)+diag(b,1)+diag(c,-1);
   v=J\d;
   y=y+[0;v;0];
   if norm(v)<tol
      disp([x',w,y]);
      disp([k,norm(y-w)]);
      disp('   xito');break
   end
   k=k+1;
end
if k>m
   disp('excede # iteraciones');
end
%  Las siguientes instrucciones muestran una grfica de la solucin
%  con las aproximaciones obtenidas
subplot(1,2,1);
plot(x,y,'r');
xlabel t;
%ylabel('Exacta (*) y calculada (-)');
title('PVF no lineal')
hold on
fplot(wun,[p1 p2],'k*');
zoom,hold off
%print -deps2 .\fig\pvfDFnl.eps
subplot(1,2,2);

% Grfica de los errores
ee=w-y;
plot(x,ee,'k');
xlabel t,title('Error');
%print -deps2 .\fig\pvfDFnl.eps